/**********************************************************************
 * Copyright (c) 2005, 2012 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: TreeLabelProvider.java,v 1.3 2005-04-21 17:53:03 curtispd Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.trace.ui.internal.util;

import java.util.Hashtable;
import java.util.Iterator;

import org.eclipse.hyades.trace.ui.launcher.IProfilingSetType;
import org.eclipse.hyades.trace.ui.launcher.IProfilingSetTypeGroup;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;


public class TreeLabelProvider extends LabelProvider {
	private Hashtable<ImageDescriptor, Image> iconList;

	public TreeLabelProvider(ImageDescriptor[] descList) {
		iconList = new Hashtable<ImageDescriptor, Image>();
		for(int i = 0; i < descList.length; i ++) {
			if (descList[i] != null && iconList.containsKey(descList[i]) == false) {
				iconList.put(descList[i], descList[i].createImage());
			}
		}
	}

	public String getText(Object element) {
		if (element instanceof IProfilingSetType) {
			return ((IProfilingSetType)element).getName();
		} else if(element instanceof IProfilingSetTypeGroup){
			return ((IProfilingSetTypeGroup)element).getName();
		}
		return element.toString();
	}

	public Image getImage(Object element) {
		ImageDescriptor imageDesc = null;
		if (element instanceof IProfilingSetType) {
			imageDesc = ((IProfilingSetType)element).getImage();
		}
		else if(element instanceof IProfilingSetTypeGroup){
			imageDesc = ((IProfilingSetTypeGroup)element).getIcon();
		}			
		if(imageDesc == null) {
			return null;
		}
		
		return iconList.get(imageDesc); 
	}

	public void dispose() {
		Iterator<ImageDescriptor> list = iconList.keySet().iterator();
		while(list.hasNext()) {
			iconList.get(list.next()).dispose();
		}
		iconList.clear();
		iconList = null;
    }
}

