//
// This file is *NOT* build with the martini profiler. It is a test program to
// assist in sanity checking the atomic operations defined in OSA_Atomic.h.
// 
#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "OSA_Atomic.h"

#define SPINCOUNT 1000000
#define THREADCOUNT 10

static void *incWorker( void * arg ) {

        int *counter = (int*)arg;

        sleep( 1 );
        for( int i=0; i<SPINCOUNT; i++ )
                atomicInc( counter );
        return NULL;

}

static void *decWorker( void * arg ) {

        int *counter = (int*)arg;

        sleep( 1 );
        for( int i=0; i<SPINCOUNT; i++ )
                atomicDec( counter );
        return NULL;

}

static bool testInc() {

	int i, counter = 0;
	pthread_t workers[ THREADCOUNT ];
	
	fprintf( stderr, "THREADCOUNT = %d, SPINCOUNT = %d, THREADCOUNT * SPINCOUNT = %d\n", 
	         THREADCOUNT, SPINCOUNT, THREADCOUNT * SPINCOUNT );
	fprintf( stderr, "counter = %d\n", counter );
	
	for( i=0; i<THREADCOUNT; i++ )
		pthread_create( &workers[i], NULL, incWorker, &counter );
	
	
	for( int i=0; i<THREADCOUNT; i++ )
		pthread_join( workers[i], NULL );
	
	fprintf( stderr, "counter = %d\n", counter );
	
	return (THREADCOUNT * SPINCOUNT) == counter;

}

static bool testDec() {

	int i, counter = THREADCOUNT * SPINCOUNT;
	pthread_t workers[ THREADCOUNT ];
	
	fprintf( stderr, "THREADCOUNT = %d, SPINCOUNT = %d, THREADCOUNT * SPINCOUNT = %d\n", 
	         THREADCOUNT, SPINCOUNT, THREADCOUNT * SPINCOUNT );
	fprintf( stderr, "counter = %d\n", counter );
	
	for( i=0; i<THREADCOUNT; i++ )
		pthread_create( &workers[i], NULL, decWorker, &counter );
	
	
	for( int i=0; i<THREADCOUNT; i++ )
		pthread_join( workers[i], NULL );
	
	fprintf( stderr, "counter = %d\n", counter );	
	return 0 == counter;

}

#define TEST( tst ) \
	printf( "%s : " #tst "\n", tst() ? "PASS" : "FAIL" )

int main( int argc, char *argv[] ) {

	TEST( testInc );
	TEST( testDec );

	return 0;

}
